rm(list=ls())
library(stringr)

#Read in data
data <- read.csv("../1_Input/resultphyschem.csv")

sites <- unique(data[c("MonitoringLocationIdentifier", "MonitoringLocationName")])
sites$order <- ""
sites$include <- "Yes"

write.csv(sites, "../3_IntermediateOutput/SitesList.csv", row.names = FALSE)

#Replace retired names
data$CharacteristicName[data$CharacteristicName == 
                          "Nutrient-nitrogen***retired***use TOTAL NITROGEN, MIXED FORMS with speciation AS N"] <-
  "Total Nitrogen, mixed forms"

data$CharacteristicName[data$CharacteristicName == 
                          "Inorganic nitrogen (nitrate and nitrite) ***retired***use Nitrate + Nitrite"] <-
  "Nitrate + Nitrite"

#Drop results that are "Not Reported" 
notreport <- data[c(which(data$ResultDetectionConditionText == "Not Reported")) ,]

data <- data[!data$ResultDetectionConditionText == "Not Reported" ,]

if(length(notreport$OrganizationIdentifier > 0)){
  
  write.csv(notreport, "../3_IntermediateOutput/NotReported_DroppedFromUpload.csv")
  
}

#Standardize Units
if(any(str_detect(data$ResultMeasure.MeasureUnitCode, "mg/L"))&
   any(str_detect(data$ResultMeasure.MeasureUnitCode, "ug/L"))){
  
  data$ResultMeasureValue[data$ResultMeasure.MeasureUnitCode == "mg/L"] <- data$ResultMeasureValue[data$ResultMeasure.MeasureUnitCode == "mg/L"]*1000
  data$ResultMeasure.MeasureUnitCode[data$ResultMeasure.MeasureUnitCode == "mg/L"] <- "ug/L"
  
}


#Create Boxplots ####

#If interested in manually reordering boxplots... (otherwise default is alphabetical)
#unique(data$CharacteristicName)


#Make filename
data$filename <- data$CharacteristicName
data$filename <- sub("/", "-", data$filename)
data$filename <- sub("/", "-", data$filename)

#get units
units <- unique(data[c("CharacteristicName", 
                       "ResultMeasure.MeasureUnitCode", 
                       "ResultDetectionConditionText",
                       "filename")])

units <- units[-which(units$ResultDetectionConditionText == "Not Detected") ,]
units <- units[, -3]


for(i in 1:length(data$ResultMeasureValue)){
  if(data$ResultDetectionConditionText[i] == "Not Detected"
     | data$ResultMeasureValue[i] == "ND"){
    data$ResultDetectionConditionText[i] <- "Not Detected"
    data$ResultMeasureValue[i] <- 0
   # data$pch[i] <- 8
  }
}


data$ResultMeasureValue <- as.numeric(data$ResultMeasureValue)

ifelse(str_detect(data$ActivityStartDate, "-"),
       yes= data$yr <- format(as.POSIXct(data$ActivityStartDate), format= "%Y"),
       no= data$yr <- format(as.Date(data$ActivityStartDate, format= "%m/%d/%Y"),
                             format= "%Y"))

data <- data[data$ActivityTypeCode == "Sample-Routine"
             | data$ActivityTypeCode == "Field Msr/Obs" ,]


#create boxplots
for(i in 1:length(units$CharacteristicName)){
  
  data.temp <- data[which(data$CharacteristicName 
                            == units$CharacteristicName[i]) ,]
  
  data.temp$MonitoringLocationIdentifier <- sub("MTWTRSHD_WQX-", "",
                                                data.temp$MonitoringLocationIdentifier)
  
  data.temp$MonitoringLocationIdentifier <- sub("MTVOLWQM_WQX-", "",
                                                data.temp$MonitoringLocationIdentifier)
  
  yr.range <- c(min(data.temp$yr), max(data.temp$yr))
  
  ymax <- max(data.temp$ResultMeasureValue[is.na(data.temp$ResultMeasureValue) == FALSE])
  ymin <- min(data.temp$ResultMeasureValue[is.na(data.temp$ResultMeasureValue) == FALSE])

  #Single year boxplot
  png(filename = paste0("../3_IntermediateOutput/Plots/AllYears_",
                        units$filename[i], "_boxplot.png"),
      width= 800, height= 800)
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    par(mar= c(7, 4, 4, 2))
    
  }else{
    
    par(mar= c(9, 4, 4, 2))
  }

  
  #Boxplot
  boxplot(data.temp$ResultMeasureValue ~ data.temp$MonitoringLocationIdentifier,
          main= paste(units$CharacteristicName[i], yr.range[1], "to", yr.range[2]),
          xlab= "",
          ylab= units$ResultMeasure.MeasureUnitCode[i],
          cex.axis= 0.75,
          las=2,
          ylim= c(ymin, ymax+ymax*0.1))
  
  #Summarize data
  data.site.sample <- aggregate(data.temp$ResultMeasureValue,
                                by=list(data.temp$MonitoringLocationIdentifier), 
                                length)
  
  colnames(data.site.sample) <- c("MonitoringLocationIdentifier", "SampleCount")
  
  for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
    
    data.site.sample$median[j] <- round(median(data.temp$ResultMeasureValue
                                               [data.temp$MonitoringLocationIdentifier == data.site.sample$MonitoringLocationIdentifier[j]]), 3)
    
  }
  
  data.temp$ndcount <- ifelse(data.temp$ResultDetectionConditionText == "Not Detected",
                              yes= 1,
                              no= 0)
  
  ndcount <- aggregate(data.temp$ndcount,
                       by=list(data.temp$MonitoringLocationIdentifier), 
                       sum)
  colnames(ndcount) <- c("MonitoringLocationIdentifier", "ndcount")
  
  data.site.sample <- merge(data.site.sample, ndcount, by= "MonitoringLocationIdentifier")
   
  data.site.sample$position <- c(1:length(data.site.sample$MonitoringLocationIdentifier))
  
  #data.x <- seq(from=1, to=length(data.site.sample$Site))
  
  
  nd <- data.temp[data.temp$ResultDetectionConditionText == "Not Detected" 
                | is.na(data.temp$ResultDetectionConditionText) == TRUE,]
  yd <- data.temp[data.temp$ResultDetectionConditionText != "Not Detected" 
                | is.na(data.temp$ResultDetectionConditionText) == TRUE,]
  
  nd.missing <- which(unique(nd$MonitoringLocationIdentifier) %in% unique(yd$MonitoringLocationIdentifier) == FALSE)
  
  if(length(nd.missing) > 0){
    nd.row <- NULL
    
    for(j in 1:length(nd.missing)){
      site <- unique(nd$MonitoringLocationIdentifier)[nd.missing[j]]
      row <- c(rep(NA, times= which(colnames(yd) == 
                                      "MonitoringLocationIdentifier")-1),
               site, rep(NA, times= ncol(yd)-
                           which(colnames(yd) == 
                                   "MonitoringLocationIdentifier")))
      nd.row <- rbind(nd.row, row)
    }
    
    colnames(nd.row) <- colnames(yd)
    rownames(nd.row) <- NULL
    yd <- rbind(yd, nd.row)
    
    
  }
  
  yd.missing <- which(unique(yd$MonitoringLocationIdentifier) %in% unique(nd$MonitoringLocationIdentifier) == FALSE)
  
  if(length(yd.missing) > 0){
    yd.row <- NULL
    
    for(j in 1:length(yd.missing)){
      site <- unique(yd$MonitoringLocationIdentifier)[yd.missing[j]]
      row <- c(rep(NA, times= which(colnames(nd) == 
                                      "MonitoringLocationIdentifier")-1),
               site, rep(NA, times= ncol(nd)-
                           which(colnames(nd) == 
                                   "MonitoringLocationIdentifier")))
      yd.row <- rbind(yd.row, row)
    }
    colnames(yd.row) <- colnames(nd)
    rownames(yd.row) <- NULL
    nd <- rbind(nd, yd.row)
  }
  
  yd.order <- with(yd, reorder(yd$MonitoringLocationIdentifier,
                               yd$MonitoringLocationIdentifier))
  
  
  nd.order <- with(nd, reorder(nd$MonitoringLocationIdentifier,
                               nd$MonitoringLocationIdentifier))
  
  
  stripchart(yd$ResultMeasureValue ~ yd.order,
             method= "jitter",
             jitter= 0,
             add= TRUE,
             vertical= TRUE,
             pch= 16,
             cex = 0.9)
  
  stripchart(nd$ResultMeasureValue ~ nd.order,
             method= "jitter",
             jitter= 0,
             add= TRUE,
             vertical= TRUE,
             pch= 8,
             cex = 0.9)
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    mtext(text= "Samples:", side=1, line= 6, at= 0, cex= 0.85 )
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      mtext(at=data.site.sample$position[j], side=1, line=6, text= data.site.sample$SampleCount[j], cex=0.85)
      
    }
    
  }else{
    
    mtext(text= "Samples:", side=1, line= 8, at= 0, cex= 0.85 )
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      mtext(at=data.site.sample$position[j], side=1, line=8, text= data.site.sample$SampleCount[j], cex=0.85)
      
    
  }
  }
  
  text(labels= "Median:", x= 0.54, y= ymax+ymax*0.12, cex= 0.85 )
  
  for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
    text(x=data.site.sample$position[j], y=ymax+ymax*0.1, labels= data.site.sample$median[j], cex=0.85)
    
  }
  
  if(data.temp$ActivityTypeCode[1] == "Sample-Routine"){
    
    text(labels= "Non-Detects:", x= 0.71, y= ymax+ymax*0.05, cex= 0.85 )
    
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      text(x=data.site.sample$position[j], y=ymax+ymax*0.03, labels= data.site.sample$ndcount[j], cex=0.85)
      
    }
  }
  
  
  dev.off()
  
  
  #Individual year boxplots  
  years <- unique(data.temp$yr)
  
  pdf(file= paste0("../3_IntermediateOutput/Plots/IndividualYear_",
                   units$filename[i], "_boxplot.pdf"),
      width = 8.5, height= 11)
  
  par(mfrow= c(2,1))
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    par(mar= c(7, 4, 4, 2))
    
  }else{
    
    par(mar= c(9, 4, 4, 2))
  }
  for(y in 1:length(years)){
    
    yr <- years[y]
    
    data.yr <- data.temp[data.temp$yr == yr ,]
    
    if(length(data.yr$OrganizationIdentifier) > 0){
      
      #Summarize data
      data.site.sample <- aggregate(data.yr$ResultMeasureValue,
                                    by=list(data.yr$MonitoringLocationIdentifier), 
                                    length)
      
      colnames(data.site.sample) <- c("MonitoringLocationIdentifier", "SampleCount")
      
      for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
        
        data.site.sample$median[j] <- round(median(data.yr$ResultMeasureValue
                                                   [data.yr$MonitoringLocationIdentifier == data.site.sample$MonitoringLocationIdentifier[j]]), 3)
        
      }
      
      data.yr$ndcount <- ifelse(data.yr$ResultDetectionConditionText == "Not Detected",
                                yes= 1,
                                no= 0)
      
      ndcount <- aggregate(data.yr$ndcount,
                           by=list(data.yr$MonitoringLocationIdentifier), 
                           sum)
      colnames(ndcount) <- c("MonitoringLocationIdentifier", "ndcount")
      
      data.site.sample <- merge(data.site.sample, ndcount, by= "MonitoringLocationIdentifier")
      
      data.site.sample$position <- c(1:length(data.site.sample$MonitoringLocationIdentifier))
      
      #data.x <- seq(from=1, to=length(data.site.sample$Site))
      
      #Plot data
      boxplot(data.yr$ResultMeasureValue ~ data.yr$MonitoringLocationIdentifier,
              main= paste(units$CharacteristicName[i], yr),
              xlab= "",
              ylab= units$ResultMeasure.MeasureUnitCode[i],
              cex.axis= 0.75,
              las=2,
              ylim= c(ymin, ymax+ymax*0.1)
      )
      
      nd <- data.yr[data.yr$ResultDetectionConditionText == "Not Detected" 
                        | is.na(data.yr$ResultDetectionConditionText) == TRUE,]
      yd <- data.yr[data.yr$ResultDetectionConditionText != "Not Detected" 
                        | is.na(data.yr$ResultDetectionConditionText) == TRUE,]
      
      nd.missing <- which(unique(nd$MonitoringLocationIdentifier) %in% unique(yd$MonitoringLocationIdentifier) == FALSE)
      
      if(length(nd.missing) > 0){
        nd.row <- NULL
        
        for(j in 1:length(nd.missing)){
          site <- unique(nd$MonitoringLocationIdentifier)[nd.missing[j]]
          row <- c(rep(NA, times= which(colnames(yd) == 
                                          "MonitoringLocationIdentifier")-1),
                   site, rep(NA, times= ncol(yd)-
                               which(colnames(yd) == 
                                       "MonitoringLocationIdentifier")))
          nd.row <- rbind(nd.row, row)
        }
        
        colnames(nd.row) <- colnames(yd)
        rownames(nd.row) <- NULL
        yd <- rbind(yd, nd.row)
        
        
      }
      
      yd.missing <- which(unique(yd$MonitoringLocationIdentifier) %in% unique(nd$MonitoringLocationIdentifier) == FALSE)
      
      if(length(yd.missing) > 0){
        yd.row <- NULL
        
        for(j in 1:length(yd.missing)){
          site <- unique(yd$MonitoringLocationIdentifier)[yd.missing[j]]
          row <- c(rep(NA, times= which(colnames(nd) == 
                                          "MonitoringLocationIdentifier")-1),
                   site, rep(NA, times= ncol(nd)-
                               which(colnames(nd) == 
                                       "MonitoringLocationIdentifier")))
          yd.row <- rbind(yd.row, row)
        }
        colnames(yd.row) <- colnames(nd)
        rownames(yd.row) <- NULL
        nd <- rbind(nd, yd.row)
      }
      
      yd.order <- with(yd, reorder(yd$MonitoringLocationIdentifier,
                                   yd$MonitoringLocationIdentifier))
      
  
      nd.order <- with(nd, reorder(nd$MonitoringLocationIdentifier,
                                   nd$MonitoringLocationIdentifier))
      
      
      stripchart(yd$ResultMeasureValue ~ yd.order,
                 method= "jitter",
                 jitter= 0,
                 add= TRUE,
                 vertical= TRUE,
                 pch= 16,
                 cex = 0.9)
      
      stripchart(nd$ResultMeasureValue ~ nd.order,
                 method= "jitter",
                 jitter= 0,
                 add= TRUE,
                 vertical= TRUE,
                 pch= 8,
                 cex = 0.9)
      
      if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
        
        mtext(text= "Samples:", side=1, line= 6, at= 0, cex= 0.85 )
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          mtext(at=data.site.sample$position[j], side=1, line=6, text= data.site.sample$SampleCount[j], cex=0.85)
          
        }
        
      }else{
        
        mtext(text= "Samples:", side=1, line= 8, at= 0, cex= 0.85 )
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          mtext(at=data.site.sample$position[j], side=1, line=8, text= data.site.sample$SampleCount[j], cex=0.85)
          
          
        }
      }
      text(labels= "Median:", x= 0.54, y= ymax+ymax*0.12, cex= 0.85 )
      
      for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
        text(x=data.site.sample$position[j], y=ymax+ymax*0.07, labels= data.site.sample$median[j], cex=0.85)
        
      }
      
      if(data.yr$ActivityTypeCode[1] == "Sample-Routine"){
        
        text(labels= "Non-Detects:", x= 0.85, y= ymax+ymax*0.01, cex= 0.85 )
        
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          text(x=data.site.sample$position[j], y=ymax-ymax*0.03, labels= data.site.sample$ndcount[j], cex=0.85)
          
        }
      }
     
      
    }
    
    #if(length(data.yr$OrganizationIdentifier) == 0){
      
    #  plot(NULL,
     #      main= paste(wqx.variables$CharacteristicName[i], yr),
      #     ylim= c(ymin, ymax+ymax*0.1),
       #    xlim= c(0,1),
        #   xaxt= "n",
         #  xlab= "",
          # ylab= ""
      #)
      
    #}
    
    
  }
  
  dev.off()
  
}

#Create Parameter List
params <- data.frame(CharacteristicName= unique(data$CharacteristicName),
                     CurrentYmin= "",
                     CurrentYmax= "",
                     NewYmin= "",
                     NewYmax= "")

for(i in 1:length(params$CharacteristicName)){
  
  params$CurrentYmin[i] <- min(data$ResultMeasureValue[data$CharacteristicName == params$CharacteristicName[i]])
  params$CurrentYmax[i] <- max(data$ResultMeasureValue[data$CharacteristicName == params$CharacteristicName[i]])
  
  
}

write.csv(params, "../3_IntermediateOutput/YAxisLimits.csv", row.names = FALSE)







