rm(list=ls())
library(stringr)

#Read in data
data <- read.csv("../1_Input/resultphyschem.csv")
sites <- read.csv("../3_IntermediateOutput/SitesListUpdated.csv")
params <- read.csv("../3_IntermediateOutput/YAxisLimitsUpdated.csv")

data <- merge(data, sites, by= c("MonitoringLocationIdentifier",
                                 "MonitoringLocationName"))

data <- data[data$include == "Yes" ,]

data<- merge(data, params, by= "CharacteristicName")

#Replace retired names
data$CharacteristicName[data$CharacteristicName == 
                          "Nutrient-nitrogen***retired***use TOTAL NITROGEN, MIXED FORMS with speciation AS N"] <-
  "Total Nitrogen, mixed forms"

data$CharacteristicName[data$CharacteristicName == 
                          "Inorganic nitrogen (nitrate and nitrite) ***retired***use Nitrate + Nitrite"] <-
  "Nitrate + Nitrite"

#Remove Not Reported
data <- data[!data$ResultDetectionConditionText == "Not Reported" ,]

#Standardize Units
if(any(str_detect(data$ResultMeasure.MeasureUnitCode, "mg/L"))&
   any(str_detect(data$ResultMeasure.MeasureUnitCode, "ug/L"))){
  
  data$ResultMeasureValue[data$ResultMeasure.MeasureUnitCode == "mg/L"] <- data$ResultMeasureValue[data$ResultMeasure.MeasureUnitCode == "mg/L"]*1000
  data$ResultMeasure.MeasureUnitCode[data$ResultMeasure.MeasureUnitCode == "mg/L"] <- "ug/L"
  
}

#Make sites list match boxplot names
sites$MonitoringLocationIdentifier <- sub("MTWTRSHD_WQX-", "",
                                          sites$MonitoringLocationIdentifier)

sites$MonitoringLocationIdentifier <- sub("MTVOLWQM_WQX-", "",
                                          sites$MonitoringLocationIdentifier)


#Create Boxplots ####

#Make filename
data$filename <- data$CharacteristicName
data$filename <- sub("/", "-", data$filename)
data$filename <- sub("/", "-", data$filename)

#get units
units <- unique(data[c("CharacteristicName", 
                       "ResultMeasure.MeasureUnitCode", 
                       "ResultDetectionConditionText",
                       "filename")])

units <- units[-which(units$ResultDetectionConditionText == "Not Detected") ,]
units <- units[, -3]


for(i in 1:length(data$ResultMeasureValue)){
  if(data$ResultDetectionConditionText[i] == "Not Detected"
     | data$ResultMeasureValue[i] == "ND"){
    data$ResultDetectionConditionText[i] <- "Not Detected"
    data$ResultMeasureValue[i] <- 0
    # data$pch[i] <- 8
  }
}

data$ResultMeasureValue <- as.numeric(data$ResultMeasureValue)

ifelse(str_detect(data$ActivityStartDate, "-"),
       yes= data$yr <- format(as.POSIXct(data$ActivityStartDate), format= "%Y"),
       no= data$yr <- format(as.Date(data$ActivityStartDate, format= "%m/%d/%Y"),
                             format= "%Y"))

data <- data[data$ActivityTypeCode == "Sample-Routine"
             | data$ActivityTypeCode == "Field Msr/Obs" ,]

#create boxplots
for(i in 1:length(units$CharacteristicName)){
  
  data.temp <- data[which(data$CharacteristicName 
                          == units$CharacteristicName[i]) ,]
  
  data.temp$MonitoringLocationIdentifier <- sub("MTWTRSHD_WQX-", "",
                                                data.temp$MonitoringLocationIdentifier)
  
  data.temp$MonitoringLocationIdentifier <- sub("MTVOLWQM_WQX-", "",
                                                data.temp$MonitoringLocationIdentifier)
  
  yr.range <- c(min(data.temp$yr), max(data.temp$yr))

  ymin <- unique(data.temp$NewYmin)
  ymax <- unique(data.temp$NewYmax)
  
  #Create order
  data.order <- with(data.temp, reorder(data.temp$MonitoringLocationIdentifier, 
                                   data.temp$order, na.rm=FALSE))
  
  
  #Single year boxplot
  png(filename = paste0("../4_FinalOutput/AllYears_",
                        units$filename[i], "_boxplot.png"),
      width= 800, height= 800)
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    par(mar= c(7, 4, 4, 2))
    
  }else{
    
    par(mar= c(9, 4, 4, 2))
  }
  
  #Boxplot
  boxplot(data.temp$ResultMeasureValue ~ data.order,
          main= paste(units$CharacteristicName[i], yr.range[1], "to", yr.range[2]),
          xlab= "",
          ylab= units$ResultMeasure.MeasureUnitCode[i],
          cex.axis= 0.75,
          las=2,
          ylim= c(ymin, ymax+ymax*0.1))
  
  #Summarize data
  data.site.sample <- aggregate(data.temp$ResultMeasureValue,
                                by=list(data.temp$MonitoringLocationIdentifier), 
                                length)
  
  colnames(data.site.sample) <- c("MonitoringLocationIdentifier", "SampleCount")
  
  for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
    
    data.site.sample$median[j] <- round(median(data.temp$ResultMeasureValue
                                               [data.temp$MonitoringLocationIdentifier == data.site.sample$MonitoringLocationIdentifier[j]]), 3)
    
  }
  
  data.temp$ndcount <- ifelse(data.temp$ResultDetectionConditionText == "Not Detected",
                            yes= 1,
                            no= 0)
  
  ndcount <- aggregate(data.temp$ndcount,
                       by=list(data.temp$MonitoringLocationIdentifier), 
                       sum)
  colnames(ndcount) <- c("MonitoringLocationIdentifier", "ndcount")
  
  data.site.sample <- merge(data.site.sample, ndcount, by= "MonitoringLocationIdentifier")
  
  order.data <- unique(data.temp[c("MonitoringLocationIdentifier", "order")])
  
  data.site.sample <- merge(data.site.sample, order.data, by= "MonitoringLocationIdentifier")
  
  data.site.sample <- data.site.sample[order(data.site.sample$order) ,]
  
  data.site.sample$position <- c(1:length(data.site.sample$order))
  
  #data.x <- unique(data.temp$order)
  
  # if(all(seq(1:length(data.x)) %in% data.x) == FALSE){
  #   
  #   which.missing <- which(seq(1:length(data.x)) %in% data.x == FALSE)
  #   diff <- max(data.x)-length(data.x)
  #   data.x[data.x > which.missing] <- data.x[data.x > which.missing]-diff
  #   
  # }
  # 
  # if(length(data.x) < max(data.x)){
  #   
  #   diff <- max(data.x)-length(data.x)
  #   
  #   data.x[data.x > length(data.x)] <- data.x[data.x > length(data.x)]-diff
  #   
  # }
  # 
  nd <- data.temp[data.temp$ResultDetectionConditionText == "Not Detected" 
                  | is.na(data.temp$ResultDetectionConditionText) == TRUE,]
  yd <- data.temp[data.temp$ResultDetectionConditionText != "Not Detected" 
                  | is.na(data.temp$ResultDetectionConditionText) == TRUE,]
  
  nd.missing <- which(unique(nd$MonitoringLocationIdentifier) %in% unique(yd$MonitoringLocationIdentifier) == FALSE)
  
  if(length(nd.missing) > 0){
    nd.row <- NULL
    
    for(j in 1:length(nd.missing)){
      site <- unique(nd$MonitoringLocationIdentifier)[nd.missing[j]]
      row <- c(rep(NA, times= which(colnames(yd) == 
                                      "MonitoringLocationIdentifier")-1),
               site, rep(NA, times= ncol(yd)-
                           which(colnames(yd) == 
                                   "MonitoringLocationIdentifier")))
      nd.row <- rbind(nd.row, row)
    }
    
    colnames(nd.row) <- colnames(yd)
    rownames(nd.row) <- NULL
    yd <- rbind(yd, nd.row)
    
    
  }
  
  yd.missing <- which(unique(yd$MonitoringLocationIdentifier) %in% unique(nd$MonitoringLocationIdentifier) == FALSE)
  
  if(length(yd.missing) > 0){
    yd.row <- NULL
    
    for(j in 1:length(yd.missing)){
      site <- unique(yd$MonitoringLocationIdentifier)[yd.missing[j]]
      row <- c(rep(NA, times= which(colnames(nd) == 
                                      "MonitoringLocationIdentifier")-1),
               site, rep(NA, times= ncol(nd)-
                           which(colnames(nd) == 
                                   "MonitoringLocationIdentifier")))
      yd.row <- rbind(yd.row, row)
    }
    colnames(yd.row) <- colnames(nd)
    rownames(yd.row) <- NULL
    nd <- rbind(nd, yd.row)
  }

  
  for(k in 1:length(yd$MonitoringLocationIdentifier)){
    
    if(is.na(yd$order[k])){
      
      yd$order[k] <- sites[which(sites$MonitoringLocationIdentifier == yd$MonitoringLocationIdentifier[k]),
                           "order"]
      
    }
    
  }
  
  for(k in 1:length(nd$MonitoringLocationIdentifier)){
    
    if(is.na(nd$order[k])){
      
      nd$order[k] <- sites[which(sites$MonitoringLocationIdentifier == nd$MonitoringLocationIdentifier[k]),
                           "order"]
      
    }
    
  }
  
  yd.order <- with(yd, reorder(yd$MonitoringLocationIdentifier,
                               as.numeric(yd$order)))
  
  
  nd.order <- with(nd, reorder(nd$MonitoringLocationIdentifier,
                               as.numeric(nd$order)))
  
  
  stripchart(yd$ResultMeasureValue ~ yd.order,
             method= "jitter",
             jitter= 0,
             add= TRUE,
             vertical= TRUE,
             pch= 16,
             cex = 0.9)
  
  stripchart(nd$ResultMeasureValue ~ nd.order,
             method= "jitter",
             jitter= 0,
             add= TRUE,
             vertical= TRUE,
             pch= 8,
             cex = 0.9)
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    mtext(text= "Samples:", side=1, line= 6, at= 0, cex= 0.85 )
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      mtext(at=data.site.sample$position[j], side=1, line=6, text= data.site.sample$SampleCount[j], cex=0.85)
      
    }
    
  }else{
    
    mtext(text= "Samples:", side=1, line= 8, at= 0, cex= 0.85 )
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      mtext(at=data.site.sample$position[j], side=1, line=8, text= data.site.sample$SampleCount[j], cex=0.85)
      
      
    }
  }
  
  
  if(ymax <= 8 | ymax >= 10){text(labels= "Median:", x= 0.54, y= ymax+ymax*0.12, cex= 0.85 )}
  
  if(ymax > 8 & ymax < 10){text(labels= "Median:", x= 0.54, y= ymax+ymax*0.1, cex= 0.85 )}
  
  for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
    text(x=data.site.sample$position[j], y=ymax+ymax*0.1, labels= data.site.sample$median[j], cex=0.85)
    
  }
  
  if(data.temp$ActivityTypeCode[1] == "Sample-Routine"){
    
    text(labels= "Non-Detects:", x= 0.62, y= ymax+ymax*0.05, cex= 0.85 )
    
    for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
      text(x=data.site.sample$position[j], y=ymax+ymax*0.03, labels= data.site.sample$ndcount[j], cex=0.85)
      
    }
  }
  
  
  dev.off()
  
  
  #Individual year boxplots  
  years <- unique(data.temp$yr)
  years <- sort(years)
  
  pdf(file= paste0("../4_FinalOutput/IndividualYear_",
                   units$filename[i], "_boxplot.pdf"),
      width = 8.5, height= 11)
  
  par(mfrow= c(2,1))
  
  if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
    
    par(mar= c(7, 4, 4, 2))
    
  }else{
    
    par(mar= c(9, 4, 4, 2))
  }
  
  for(y in 1:length(years)){
    
    yr <- years[y]
    
    data.yr <- data.temp[data.temp$yr == yr ,]
    
    if(length(data.yr$OrganizationIdentifier) > 0){
      
      data.order.2 <- with(data.yr, reorder(data.yr$MonitoringLocationIdentifier, 
                                          data.yr$order, na.rm=FALSE))
      
      
      #Summarize data
      data.site.sample <- aggregate(data.yr$ResultMeasureValue,
                                    by=list(data.yr$MonitoringLocationIdentifier), 
                                    length)
      
      colnames(data.site.sample) <- c("MonitoringLocationIdentifier", "SampleCount")
      
      for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
        
        data.site.sample$median[j] <- round(median(data.yr$ResultMeasureValue
                                                   [data.yr$MonitoringLocationIdentifier == data.site.sample$MonitoringLocationIdentifier[j]]), 3)
        
      }
      
      data.yr$ndcount <- ifelse(data.yr$ResultDetectionConditionText == "Not Detected",
                                yes= 1,
                                no= 0)
      
      ndcount <- aggregate(data.yr$ndcount,
                           by=list(data.yr$MonitoringLocationIdentifier), 
                           sum)
      colnames(ndcount) <- c("MonitoringLocationIdentifier", "ndcount")
      
      data.site.sample <- merge(data.site.sample, ndcount, by= "MonitoringLocationIdentifier")
      
      order.data <- unique(data.yr[c("MonitoringLocationIdentifier", "order")])
      
      data.site.sample <- merge(data.site.sample, order.data, by= "MonitoringLocationIdentifier")
      
      data.site.sample <- data.site.sample[order(data.site.sample$order) ,]
      
      data.site.sample$position <- c(1:length(data.site.sample$order))
      
      # if(all(seq(1:length(data.x)) %in% data.x) == FALSE){
      #   
      #   which.missing <- which(seq(1:max(data.x)) %in% data.x == FALSE)
      #   
      #   if(length(which.missing) > 1){
      #     
      #     diff.breaks <- which.missing[c(diff(which.missing)!=1, TRUE)]
      #     
      #     data.x[data.x > diff.breaks[1] & data.x < diff.breaks[2]] <- data.x[data.x > diff.breaks[1] & data.x < diff.breaks[2]]-(diff.breaks[1]-max(data.x[data.x < diff.breaks[1]]))
      #     
      #     for(l in 2:length(diff.breaks)){
      #       
      #       if(l < length(diff.breaks)){
      #         
      #         data.x[data.x > diff.breaks[l] & data.x < diff.breaks[l+1]] <- data.x[data.x > diff.breaks[l] & data.x < diff.breaks[l+1]]-(diff.breaks[l]-max(data.x[data.x < diff.breaks[l]]))
      #         
      #       }else{
      #         
      #         data.x[data.x>diff.breaks[l]] <- data.x[data.x > diff.breaks[l]]-(diff.breaks[l]-max(data.x[data.x < diff.breaks[l]]))
      #         
      #       }
      #       
      #     }
      #     
      #   }
      #   
      #   if(length(which.missing) == 1){
      #     
      #     diff <- max(data.x)-length(data.x)
      #     data.x[data.x > min(which.missing)] <- data.x[data.x > min(which.missing)]-diff
      #     
      #   }
      #   
      # }
      # 
      # if(length(data.x) < max(data.x)){
      #   
      #   diff <- max(data.x)-length(data.x)
      #   
      #   data.x[data.x > length(data.x)] <- data.x[data.x > length(data.x)]-diff
      #   
      # }
      # 
      
      #Plot data
      boxplot(data.yr$ResultMeasureValue ~ data.order.2,
              main= paste(units$CharacteristicName[i], yr),
              xlab= "",
              ylab= units$ResultMeasure.MeasureUnitCode[i],
              cex.axis= 0.75,
              las=2,
              ylim= c(ymin, ymax+ymax*0.15)
      )
      
      nd <- data.yr[data.yr$ResultDetectionConditionText == "Not Detected" 
                    | is.na(data.yr$ResultDetectionConditionText) == TRUE,]
      yd <- data.yr[data.yr$ResultDetectionConditionText != "Not Detected" 
                    | is.na(data.yr$ResultDetectionConditionText) == TRUE,]
      
      nd.missing <- which(unique(nd$MonitoringLocationIdentifier) %in% unique(yd$MonitoringLocationIdentifier) == FALSE)
      
      if(length(nd.missing) > 0){
        nd.row <- NULL
        
        for(j in 1:length(nd.missing)){
          site <- unique(nd$MonitoringLocationIdentifier)[nd.missing[j]]
          row <- c(rep(NA, times= which(colnames(yd) == 
                                          "MonitoringLocationIdentifier")-1),
                   site, rep(NA, times= ncol(yd)-
                               which(colnames(yd) == 
                                       "MonitoringLocationIdentifier")))
          nd.row <- rbind(nd.row, row)
        }
        
        colnames(nd.row) <- colnames(yd)
        rownames(nd.row) <- NULL
        yd <- rbind(yd, nd.row)
        
        
      }
      
      yd.missing <- which(unique(yd$MonitoringLocationIdentifier) %in% unique(nd$MonitoringLocationIdentifier) == FALSE)
      
      if(length(yd.missing) > 0){
        yd.row <- NULL
        
        for(j in 1:length(yd.missing)){
          site <- unique(yd$MonitoringLocationIdentifier)[yd.missing[j]]
          row <- c(rep(NA, times= which(colnames(nd) == 
                                          "MonitoringLocationIdentifier")-1),
                   site, rep(NA, times= ncol(nd)-
                               which(colnames(nd) == 
                                       "MonitoringLocationIdentifier")))
          yd.row <- rbind(yd.row, row)
        }
        colnames(yd.row) <- colnames(nd)
        rownames(yd.row) <- NULL
        nd <- rbind(nd, yd.row)
      }
      
      for(k in 1:length(yd$MonitoringLocationIdentifier)){
        
        if(is.na(yd$order[k])){
          
          yd$order[k] <- sites[which(sites$MonitoringLocationIdentifier == yd$MonitoringLocationIdentifier[k]),
                               "order"]
          
        }
        
      }
      
      for(k in 1:length(nd$MonitoringLocationIdentifier)){
        
        if(is.na(nd$order[k])){
          
          nd$order[k] <- sites[which(sites$MonitoringLocationIdentifier == nd$MonitoringLocationIdentifier[k]),
                               "order"]
          
        }
        
      }
      
      yd.order <- with(yd, reorder(yd$MonitoringLocationIdentifier,
                                   as.numeric(yd$order)))
      
      
      nd.order <- with(nd, reorder(nd$MonitoringLocationIdentifier,
                                   as.numeric(nd$order)))
      
      
      stripchart(yd$ResultMeasureValue ~ yd.order,
                 method= "jitter",
                 jitter= 0,
                 add= TRUE,
                 vertical= TRUE,
                 pch= 16,
                 cex = 0.9)
      
      stripchart(nd$ResultMeasureValue ~ nd.order,
                 method= "jitter",
                 jitter= 0,
                 add= TRUE,
                 vertical= TRUE,
                 pch= 8,
                 cex = 0.9)
      
      if(max(nchar(c(data.temp$MonitoringLocationIdentifier))) < 14){
        
        mtext(text= "Samples:", side=1, line= 6, at= 0, cex= 0.85 )
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          mtext(at=data.site.sample$position[j], side=1, line=6, text= data.site.sample$SampleCount[j], cex=0.85)
          
        }
        
      }else{
        
        mtext(text= "Samples:", side=1, line= 8, at= 0, cex= 0.85 )
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          mtext(at=data.site.sample$position[j], side=1, line=8, text= data.site.sample$SampleCount[j], cex=0.85)
          
          
        }
      }
      
      
      text(labels= "Median:", x= 0.54, y= ymax+ymax*0.16, cex= 0.85 )
      
      for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
        text(x=data.site.sample$position[j], y=ymax+ymax*0.13, labels= data.site.sample$median[j], cex=0.85)
        
      }
      
      if(data.yr$ActivityTypeCode[1] == "Sample-Routine"){
        text(labels= "Non-Detects:", x= 0.68, y= ymax+ymax*0.09, cex= 0.85 )
        
        for(j in 1:length(data.site.sample$MonitoringLocationIdentifier)){
          text(x=data.site.sample$position[j], y=ymax+ymax*0.04, labels= data.site.sample$ndcount[j], cex=0.85)
          
        }
        
      }
     
      
    }
    
    #if(length(data.yr$OrganizationIdentifier) == 0){
    
    #  plot(NULL,
    #      main= paste(wqx.variables$CharacteristicName[i], yr),
    #     ylim= c(ymin, ymax+ymax*0.1),
    #    xlim= c(0,1),
    #   xaxt= "n",
    #  xlab= "",
    # ylab= ""
    #)
    
    #}
    
    
  }
  
  dev.off()
  
}

